"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SauceConnect = void 0;
const saucelabs_1 = require("saucelabs");
const MAX_SC_START_TRIALS = 3;
/**
 * Service that can be used to create a SauceConnect tunnel automatically. This can be used
 * in case developers don't set up the tunnel using the plain SauceConnect binaries.
 */
function SauceConnect(emitter, logger) {
    const log = logger.create('launcher.SauceConnect');
    // Currently active tunnel instance. See: https://github.com/saucelabs/node-saucelabs
    // for public API.
    let activeInstancePromise = null;
    let scStartTrials = 0;
    this.establishTunnel = (seleniumCapabilities, sauceConnectOptions) => __awaiter(this, void 0, void 0, function* () {
        // In case there is already a promise for a SauceConnect tunnel, we still need to return the
        // promise because we want to make sure that the launcher can wait in case the tunnel is
        // still starting.
        if (activeInstancePromise) {
            return activeInstancePromise;
        }
        // Open a new SauceConnect tunnel.
        const api = new saucelabs_1.default(seleniumCapabilities);
        return activeInstancePromise = api.startSauceConnect(Object.assign({ 
            // Redirect all logging output to Karma's logger.
            logger: log.debug.bind(log) }, sauceConnectOptions)).catch((err) => {
            ++scStartTrials;
            /**
             * fail starting Sauce Connect eventually
             */
            if (
            /**
             * only fail for ENOENT errors due to racing condition
             * see: https://github.com/saucelabs/node-saucelabs/issues/86
             */
            !err.message.includes('ENOENT') ||
                /**
                 * or if we reached the maximum rety count
                 */
                scStartTrials >= MAX_SC_START_TRIALS) {
                throw err;
            }
            log.debug(`Failed to start Sauce Connect Proxy due to ${err.stack}`);
            log.debug(`Retrying ${scStartTrials}/${MAX_SC_START_TRIALS}`);
            return this.establishTunnel(seleniumCapabilities, sauceConnectOptions);
        });
    });
    // Close the tunnel whenever Karma emits the "exit" event. In that case, we don't need to
    // reset the state because Karma will exit completely.
    emitter.on('exit', (doneFn) => __awaiter(this, void 0, void 0, function* () {
        if (activeInstancePromise) {
            log.debug('Shutting down Sauce Connect');
            // shut down Sauce Connect once all session have been terminated
            try {
                const tunnelInstance = yield activeInstancePromise;
                yield tunnelInstance.close();
            }
            catch (err) {
                log.error(`Could not close Sauce Connect Tunnel. Failure message: ${err.stack}`);
            }
        }
        doneFn();
    }));
}
exports.SauceConnect = SauceConnect;
//# sourceMappingURL=sauceconnect.js.map